import logging
import uuid
import pathlib
import base64
from typing import Callable, Dict
import random
import graphene

from claim.gql_queries import ClaimGQLType
from core.gql.gql_mutations import mutation_on_uuids_from_filter
from tasks_management.models import TaskGroup
from .apps import ClaimSamplingConfig
from core.schema import TinyInt, OpenIMISMutation
from django.contrib.auth.models import AnonymousUser
from django.core.exceptions import ValidationError, PermissionDenied
from django.utils.translation import gettext as _

from claim.gql_mutations import ClaimCodeInputType, ClaimGuaranteeIdInputType, FeedbackInputType
from core.models.user import ClaimAdmin
from claim.models import Claim

from django.db import transaction

from .models import ClaimSamplingBatch, ClaimSamplingBatchAssignment
from .services import ClaimSamplingService

logger = logging.getLogger(__name__)


class ClaimSamplingBatchInputType(OpenIMISMutation.Input):
    percentage = graphene.Int(required=True)
    t = graphene.Int(required=True)

    status = TinyInt(required=False)
    id = graphene.Int(required=False, read_only=True)
    uuid = graphene.String(required=False)
    autogenerate = graphene.Boolean(required=False)
    insuree_id = graphene.Int(required=False)
    date_from = graphene.Date(required=False)
    date_to = graphene.Date(required=False)
    icd_id = graphene.Int(required=False)
    icd_1_id = graphene.Int(required=False)
    icd_2_id = graphene.Int(required=False)
    icd_3_id = graphene.Int(required=False)
    icd_4_id = graphene.Int(required=False)
    review_status = TinyInt(required=False)
    date_claimed = graphene.Date(required=False)
    date_processed = graphene.Date(required=False)
    health_facility_id = graphene.Int(required=False)
    refer_from_id = graphene.Int(required=False)
    refer_to_id = graphene.Int(required=False)
    batch_run_id = graphene.Int(required=False)
    category = graphene.String(max_length=1, required=False)
    visit_type = graphene.String(max_length=1, required=False)
    admin_id = graphene.Int(required=False)
    explanation = graphene.String(required=False)
    adjustment = graphene.String(required=False)
    json_ext = graphene.types.json.JSONString(required=False)
    restore = graphene.UUID(required=False)
    feedback_available = graphene.Boolean(default=False)
    feedback_status = TinyInt(required=False)
    care_type = graphene.String(required=False)

    # code = graphene.Field(ClaimCodeInputType, required=True)
    # feedback = graphene.Field(FeedbackInputType, required=False)
    # guarantee_id = ClaimGuaranteeIdInputType(required=False)


@transaction.atomic
def update_or_create_claim_sampling_batch(data, user, task_group=None):

    service = ClaimSamplingService(user)

    if data.get('uuid', None) is not None:
        return service.update(data)
    else:
        claim_sampling_batch = service.create(data, task_group)
        return claim_sampling_batch


class CreateClaimSamplingBatchMutation(OpenIMISMutation):
    """
    Create a new claim sampling batch.
    """
    __filter_handlers = {
        'services': 'services__service__code__in',
        'items': 'items__item__code__in'
    }

    _mutation_module = "claim_sampling"
    _mutation_class = "CreateClaimSamplingBatchMutation"

    class Input(OpenIMISMutation.Input):
        filters = graphene.String()
        percentage = graphene.Int(required=True)
        taskGroupUuid = graphene.String(required=False)

    @classmethod
    @mutation_on_uuids_from_filter(Claim, ClaimGQLType, 'filters', __filter_handlers)
    def async_mutate(cls, user, **data):
        try:
            if type(user) is AnonymousUser or not user.id:
                raise ValidationError(_("mutation.authentication_required"))
            # if not user.has_perms(ClaimSamplingConfig.gql_mutation_create_claim_batch_samplings_perms):
            #     raise PermissionDenied(_("unauthorized"))
            if "client_mutation_id" in data:
                data.pop('client_mutation_id')
            if "client_mutation_label" in data:
                data.pop('client_mutation_label')
            # data['audit_user_id'] = user.id_for_audit
            from core.utils import TimeUtils
            # data['validity_from'] = TimeUtils.now()
            group_id = data.get('taskGroupUuid')

            task_group = TaskGroup.objects.get(id=group_id) if group_id else None
            claim_sampling_batch = update_or_create_claim_sampling_batch(data, user, task_group)
            return None
        except Exception as exc:
            from django.conf import settings
            if settings.DEBUG:
                import traceback
                logging.debug("Error in claim sampling mutation: ", exc)
                traceback.print_exc()
            return [{
                'message': _("claim.mutation.failed_to_create_claim_sampling_batch") % {'code': data['code']},
                'detail': str(exc)}]


class UpdateClaimSamplingBatchMutation(OpenIMISMutation):
    """
    Update a claim. The claim items and services can all be updated with this call
    """
    _mutation_module = "claim_sampling"
    _mutation_class = "UpdateClaimSamplingBatchMutation"

    class Input(ClaimSamplingBatchInputType):
        pass

    @classmethod
    def async_mutate(cls, user, **data):
        try:
            if type(user) is AnonymousUser or not user.id:
                raise ValidationError(
                    _("mutation.authentication_required"))
            if not user.has_perms(ClaimSamplingConfig.gql_mutation_update_claim_batch_samplings_perms):
                raise PermissionDenied(_("unauthorized"))
            data['audit_user_id'] = user.id_for_audit
            update_or_create_claim_sampling_batch(data, user)
            return None
        except Exception as exc:
            return [{
                'message': _("claim.mutation.failed_to_update_claim_sampling_batch") % {'code': data['code']},
                'detail': str(exc)}]


class ApproveClaimSamplingBatchMutation(OpenIMISMutation):
    """
    Approve given claim batch and apply deduction rate or other parameters across all claims in given batch.
    """
    _mutation_module = "claim_sampling"
    _mutation_class = "ApproveClaimSamplingBatchMutation"

    class Input(ClaimSamplingBatchInputType):
        pass

    @classmethod
    def async_mutate(cls, user, **data):
        if not user.has_perms(ClaimSamplingConfig.gql_mutation_approve_claim_batch_samplings_perms):
            raise PermissionDenied(_("unauthorized"))
        errors = []

        return errors



import logging

from claim.models import Claim
from claim_sampling.models import ClaimSamplingBatchAssignment
from claim_sampling.services import ClaimSamplingService
from core.models import User
from core.service_signals import ServiceSignalBindType
from core.signals import bind_service_signal

from tasks_management.apps import TasksManagementConfig
from tasks_management.models import Task

logger = logging.getLogger(__name__)


def _resolve_task_any(_task: Task, _user: User):
    claim_sampling_id = _task.data['data']['uuid']
    claim_sampling_service = ClaimSamplingService(user=_user)

    rejected_from_review = claim_sampling_service\
        .extrapolate_results(claim_sampling_id)


def _resolve_task_all(_task, _user):
    # TODO for now hardcoded to any, to be updated
    _resolve_task_any(_task, _user)


def _resolve_task_n(_task, _user):
    # TODO for now hardcoded to any, to be updated
    _resolve_task_any(_task, _user)


def on_claim_sampling_resolve_task(**kwargs):
    try:
        result = kwargs.get('result', None)
        if result and result['success'] \
                and result['data']['task']['status'] == Task.Status.ACCEPTED \
                and result['data']['task']['executor_action_event'] == TasksManagementConfig.default_executor_event \
                and result['data']['task']['business_event'] == 'claim_sample_extrapolation':
            data = kwargs.get("result").get("data")
            task = Task.objects.select_related('task_group').prefetch_related('task_group__taskexecutor_set').get(
                id=data["task"]["id"])
            user = User.objects.get(id=data["user"]["id"])

            # Task only relevant for this specific source
            if task.source != 'claim_sampling':
                return

            resolvers = {
                'ALL': _resolve_task_all,
                'ANY': _resolve_task_any,
                'N': _resolve_task_n,
            }

            if task.task_group.completion_policy not in resolvers:
                logger.error("Resolving task with unknown completion_policy: %s", task.task_group.completion_policy)
                return ['Unknown completion_policy: %s' % task.task_group.completion_policy]

            resolvers[task.task_group.completion_policy](task, user)
    except Exception as e:
        logger.error("Error while executing on_task_resolve", exc_info=e)
        return [str(e)]


def bind_service_signals():
    bind_service_signal(
        'task_service.resolve_task',
        on_claim_sampling_resolve_task,
        bind_type=ServiceSignalBindType.AFTER
    )



# Generated by Django 4.2.11 on 2024-05-19 14:49

import core.fields
import datetime
import dirtyfields.dirtyfields
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import simple_history.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("claim", "0028_claimattachmenttype_claimattachment_predefined_type"),
        (
            "claim_sampling",
            "0002_remove_claimsamplingbatchassignment_claim_batch_id_and_more",
        ),
    ]

    operations = [
        migrations.CreateModel(
            name="ClaimSamplingBatch",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        db_column="UUID",
                        default=None,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "is_deleted",
                    models.BooleanField(db_column="isDeleted", default=False),
                ),
                (
                    "json_ext",
                    models.JSONField(blank=True, db_column="Json_ext", null=True),
                ),
                (
                    "date_created",
                    core.fields.DateTimeField(
                        db_column="DateCreated",
                        default=datetime.datetime.now,
                        null=True,
                    ),
                ),
                (
                    "date_updated",
                    core.fields.DateTimeField(
                        db_column="DateUpdated",
                        default=datetime.datetime.now,
                        null=True,
                    ),
                ),
                ("version", models.IntegerField(default=1)),
                ("is_completed", models.BooleanField()),
                ("is_applied", models.BooleanField()),
                (
                    "computed_value",
                    models.JSONField(blank=True, db_column="ComputedValue", null=True),
                ),
                (
                    "assigned_value",
                    models.JSONField(blank=True, db_column="AssignedValue", null=True),
                ),
                (
                    "user_created",
                    models.ForeignKey(
                        db_column="UserCreatedUUID",
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="%(class)s_user_created",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "user_updated",
                    models.ForeignKey(
                        db_column="UserUpdatedUUID",
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="%(class)s_user_updated",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(dirtyfields.dirtyfields.DirtyFieldsMixin, models.Model),
        ),
        migrations.CreateModel(
            name="HistoricalClaimSamplingBatchAssignment",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        db_column="UUID", db_index=True, default=None, editable=False
                    ),
                ),
                (
                    "is_deleted",
                    models.BooleanField(db_column="isDeleted", default=False),
                ),
                (
                    "json_ext",
                    models.JSONField(blank=True, db_column="Json_ext", null=True),
                ),
                (
                    "date_created",
                    core.fields.DateTimeField(
                        db_column="DateCreated",
                        default=datetime.datetime.now,
                        null=True,
                    ),
                ),
                (
                    "date_updated",
                    core.fields.DateTimeField(
                        db_column="DateUpdated",
                        default=datetime.datetime.now,
                        null=True,
                    ),
                ),
                ("version", models.IntegerField(default=1)),
                (
                    "status",
                    models.CharField(
                        choices=[("R", "Reviewed"), ("S", "Skipped"), ("I", "Idle")],
                        default="I",
                        max_length=2,
                    ),
                ),
                ("history_id", models.AutoField(primary_key=True, serialize=False)),
                ("history_date", models.DateTimeField(db_index=True)),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "claim_batch_id",
                    models.ForeignKey(
                        blank=True,
                        db_column="ClaimSamplingBatchID",
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="claim_sampling.claimsamplingbatch",
                    ),
                ),
                (
                    "claim_id",
                    models.ForeignKey(
                        blank=True,
                        db_column="ClaimID",
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="claim.claim",
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "user_created",
                    models.ForeignKey(
                        blank=True,
                        db_column="UserCreatedUUID",
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "user_updated",
                    models.ForeignKey(
                        blank=True,
                        db_column="UserUpdatedUUID",
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "historical claim sampling batch assignment",
                "verbose_name_plural": "historical claim sampling batch assignments",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": ("history_date", "history_id"),
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="HistoricalClaimSamplingBatch",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        db_column="UUID", db_index=True, default=None, editable=False
                    ),
                ),
                (
                    "is_deleted",
                    models.BooleanField(db_column="isDeleted", default=False),
                ),
                (
                    "json_ext",
                    models.JSONField(blank=True, db_column="Json_ext", null=True),
                ),
                (
                    "date_created",
                    core.fields.DateTimeField(
                        db_column="DateCreated",
                        default=datetime.datetime.now,
                        null=True,
                    ),
                ),
                (
                    "date_updated",
                    core.fields.DateTimeField(
                        db_column="DateUpdated",
                        default=datetime.datetime.now,
                        null=True,
                    ),
                ),
                ("version", models.IntegerField(default=1)),
                ("is_completed", models.BooleanField()),
                ("is_applied", models.BooleanField()),
                (
                    "computed_value",
                    models.JSONField(blank=True, db_column="ComputedValue", null=True),
                ),
                (
                    "assigned_value",
                    models.JSONField(blank=True, db_column="AssignedValue", null=True),
                ),
                ("history_id", models.AutoField(primary_key=True, serialize=False)),
                ("history_date", models.DateTimeField(db_index=True)),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "user_created",
                    models.ForeignKey(
                        blank=True,
                        db_column="UserCreatedUUID",
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "user_updated",
                    models.ForeignKey(
                        blank=True,
                        db_column="UserUpdatedUUID",
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "historical claim sampling batch",
                "verbose_name_plural": "historical claim sampling batchs",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": ("history_date", "history_id"),
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="ClaimSamplingBatchAssignment",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        db_column="UUID",
                        default=None,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "is_deleted",
                    models.BooleanField(db_column="isDeleted", default=False),
                ),
                (
                    "json_ext",
                    models.JSONField(blank=True, db_column="Json_ext", null=True),
                ),
                (
                    "date_created",
                    core.fields.DateTimeField(
                        db_column="DateCreated",
                        default=datetime.datetime.now,
                        null=True,
                    ),
                ),
                (
                    "date_updated",
                    core.fields.DateTimeField(
                        db_column="DateUpdated",
                        default=datetime.datetime.now,
                        null=True,
                    ),
                ),
                ("version", models.IntegerField(default=1)),
                (
                    "status",
                    models.CharField(
                        choices=[("R", "Reviewed"), ("S", "Skipped"), ("I", "Idle")],
                        default="I",
                        max_length=2,
                    ),
                ),
                (
                    "claim_batch_id",
                    models.ForeignKey(
                        db_column="ClaimSamplingBatchID",
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="ClaimSamplingBatchID",
                        to="claim_sampling.claimsamplingbatch",
                    ),
                ),
                (
                    "claim_id",
                    models.ForeignKey(
                        db_column="ClaimID",
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="ClaimID",
                        to="claim.claim",
                    ),
                ),
                (
                    "user_created",
                    models.ForeignKey(
                        db_column="UserCreatedUUID",
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="%(class)s_user_created",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "user_updated",
                    models.ForeignKey(
                        db_column="UserUpdatedUUID",
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="%(class)s_user_updated",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(dirtyfields.dirtyfields.DirtyFieldsMixin, models.Model),
        ),
    ]



import random
import uuid
from typing import List

from claim.apps import ClaimConfig
from claim.models import (
    Claim, ClaimItem, ClaimService, 
)
    
from enum import Enum
from django.db.models import (
    OuterRef, Subquery, Avg, Q, Sum, F, ExpressionWrapper, 
    FloatField, DecimalField,  Subquery, OuterRef, Case, Value, When
)
from django.db.models.functions import Coalesce
from django.db import transaction
from django.utils.translation import gettext as _

from claim.services import (
    set_claims_status, update_claims_dedrems, processing_claim,
)
from claim.subqueries import (   
    total_srv_adjusted_exp, total_itm_adjusted_exp,
    total_srv_approved_exp, total_itm_approved_exp,elm_approved_exp,update_claim_approved, elm_adjusted_exp,elm_approved_exp
)
from claim_sampling.models import (
    ClaimSamplingBatch,
    ClaimSamplingBatchAssignment,
    ClaimSamplingBatchAssignmentStatus
)
from core.services import BaseService
from core.signals import register_service_signal
from core.validation import BaseModelValidation
from core import filter_validity
from tasks_management.apps import TasksManagementConfig
from tasks_management.models import Task, TaskGroup
from tasks_management.services import TaskService, _get_std_task_data_payload


class IndividualDataSourceValidation(BaseModelValidation):
    OBJECT_TYPE = ClaimSamplingBatch


class ClaimSamplingService(BaseService):
    OBJECT_TYPE = ClaimSamplingBatch

    @transaction.atomic
    @register_service_signal('claim_sampling_service.create')
    def create(self, obj_data, task_group: TaskGroup = None):
        """
        Creates a new sampling batch and assigns claims to it based on the specified sampling percentage.

        This method registers a service signal 'claim_sampling_service.create' and performs the following steps:
        1. Creates a new `ClaimSamplingBatch` instance.
        2. Determines which claims should be selected for review based on the given percentage.
        3. Creates assignments for each claim in the batch, tagging them with the appropriate review status.
        4. Saves all assignments in bulk to the database.

        Parameters:
            obj_data (dict): A dictionary containing:
                - 'percentage': The percentage of claims that should be selected for review (int).
                - 'uuids': A QuerySet of claim UUIDs that should be considered for sampling (QuerySet).
            task_group (TaskGroup): Task Group to which newly created task will be assigned.
        Usage:
            >>> claim_data = {'percentage': 20, 'uuids': Claim.objects.all()}
            >>> service = ClaimSamplingService(user)
            >>> service.create(claim_data)

        """
        percentage = int(obj_data.pop('percentage'))
        claim_batch_ids = obj_data.pop('uuids')  # UUIDS QuerySet

        if len(claim_batch_ids) == 0:
            raise ValueError(_("Claim List cannot be empty"))

        claim_batch_ids = self.__filter_already_assigned(claim_batch_ids=claim_batch_ids)

        if len(claim_batch_ids) == 0:
            raise ValueError(_("All claims already assigned"))

        if percentage < 1 or percentage > 100:
            raise ValueError(_("Percentage not in range (0, 100)"))

        sampling_batch_data = super().create({
            'is_completed': False,
            'is_applied': False,
            'computed_value': {},
            'assigned_value': {}
        })
        sampling_batch = ClaimSamplingBatch.objects.get(uuid=sampling_batch_data['data']['uuid'])

        is_selected_for_review = self.__choose_random_claims_for_review(claim_batch_ids.count(), percentage)

        batches = []
        for next_claim in claim_batch_ids:
            claim = Claim.objects.get(uuid=next_claim)
            should_be_reviewed = is_selected_for_review.pop()
            batches.append(ClaimSamplingBatchAssignment(
             uuid=uuid.uuid4(),
             claim=claim,
             claim_batch=sampling_batch,
             status=should_be_reviewed,
             user_created=self.user,
             user_updated=self.user
            ))
            if claim.review_status in [Claim.REVIEW_IDLE, Claim.REVIEW_NOT_SELECTED] \
                    and should_be_reviewed == ClaimSamplingBatchAssignmentStatus.IDLE:
                claim.review_status = Claim.REVIEW_SELECTED
                claim.save_history()
                claim.save()

        ClaimSamplingBatchAssignment.objects.bulk_create(batches)
        task = self._create_sampling_task(sampling_batch_data, sampling_batch, task_group)
        return sampling_batch

    @register_service_signal('claim_sampling_service.update')
    def update(self, obj_data):
        return super().update(obj_data)

    @register_service_signal('claim_sampling_service.delete')
    def delete(self, obj_data):
        return super().delete(obj_data)

    def __filter_already_assigned(self, claim_batch_ids):
        filtered_claim_batch_ids = claim_batch_ids.exclude(id__in=ClaimSamplingBatchAssignment.objects.filter(claim__uuid__in=claim_batch_ids).values("claim"))
        return filtered_claim_batch_ids

    @transaction.atomic
    def extrapolate_results(self, claim_sampling_id):
        claim_sampling = ClaimSamplingBatch.objects.get(id=claim_sampling_id)

        qs = Claim.objects.filter(assignments__claim_batch=claim_sampling, *filter_validity())
        
        deductible = qs.filter(review_status=Claim.REVIEW_DELIVERED)\
            .filter(Q(services__rejection_reason__lte=0) | Q(services__rejection_reason__isnull=True))\
            .annotate(total_srv_adjusted=total_srv_adjusted_exp)\
            .annotate(total_itm_adjusted=total_itm_adjusted_exp)\
            .annotate(total_srv_approved=total_srv_approved_exp)\
            .annotate(total_itm_approved=total_itm_approved_exp)\
            .aggregate(value=ExpressionWrapper(
                (Sum("total_srv_approved") + Sum("total_itm_approved")) /
                ( Sum("total_srv_adjusted") + Sum("total_itm_adjusted")),
                output_field=DecimalField()
            ))["value"]

        # Filter claims for extrapolation
        qs_extrapolated = qs.filter(
            assignments__status=ClaimSamplingBatchAssignmentStatus.SKIPPED, 
            review_status=Claim.REVIEW_IDLE
        )
        
        # update the items and services
        deductible = float(deductible or 0)
        

        # update service and item
        ClaimItem.objects.filter(claim__in=qs_extrapolated).update(price_approved=deductible * F("price_adjusted"))
        ClaimService.objects.filter(claim__in=qs_extrapolated).update(price_approved=deductible * F("price_adjusted"))

        update_claim_approved(qs_extrapolated, updates={'review_status': Claim.REVIEW_BYPASSED})

        errors = []
        for claim in qs.filter(status=Claim.STATUS_CHECKED):
            errors += processing_claim(claim, self.user, True)

        return errors

    def prepare_sampling_summary(self, claim_sampling_id):
        relevant_claims = self._get_sampling_claims(claim_sampling_id)
        total = relevant_claims.count()
        reviewed_delivered = relevant_claims.filter(review_status=Claim.REVIEW_DELIVERED)
        rejected_from_review = reviewed_delivered.filter(status=Claim.STATUS_REJECTED)
        return rejected_from_review, reviewed_delivered, total

    def apply_claim_item_service_deduction(self, claim, deduction_rate):
        claim_items = claim.items.all()

        for item in claim_items:
            new_claim_item = item
            if new_claim_item.price_approved:
                new_claim_item.price_approved *= (100-deduction_rate)/100
                new_claim_item.save()

        claim_services = claim.services.all()

        for service in claim_services:
            new_claim_service = service
            if new_claim_service.price_approved:
                new_claim_service.price_approved *= (100-deduction_rate)/100
                new_claim_service.save()

    def _get_sampling_claims(self, claim_sampling_id, include_skip=False):
        assigned_claims = ClaimSamplingBatchAssignment.objects.filter(claim_batch_id=claim_sampling_id)
        filters = [
            ClaimSamplingBatchAssignmentStatus.IDLE
        ]

        if include_skip:
            filters += ClaimSamplingBatchAssignmentStatus.SKIPPED

        claim_assignments = assigned_claims.filter(status__in=filters)
        relevant_claims = Claim.objects \
            .filter(id__in=claim_assignments.values_list('claim_id', flat=True).distinct())
        return relevant_claims

    def __init__(self, user, validation_class=IndividualDataSourceValidation):
        super().__init__(user, validation_class)

    def __choose_random_claims_for_review(self, total_elements: int, percentage: int):
        selected_for_review = int((percentage/100.0) * total_elements)
        not_selected = total_elements - selected_for_review

        # Ensure at least one claim is selected for review
        if selected_for_review == 0 and not_selected > 0:
            selected_for_review += 1
            not_selected -= 1

        # Create the matching number of claims
        result_list = [ClaimSamplingBatchAssignmentStatus.IDLE] * selected_for_review + \
                      [ClaimSamplingBatchAssignmentStatus.SKIPPED] * not_selected

        # Shuffle the list to randomize the order
        random.shuffle(result_list)
        return result_list

    def _create_sampling_task(self, sampling_batch_data, sampling_batch, task_group):
        return TaskService(self.user).create({
            'source': 'claim_sampling',
            'entity': sampling_batch,
            'status': Task.Status.ACCEPTED if task_group else Task.Status.RECEIVED,
            'executor_action_event': TasksManagementConfig.default_executor_event,
            'business_event': 'claim_sample_extrapolation',
            'data': _get_std_task_data_payload(sampling_batch_data),
            'task_group': task_group
        })

    def _update_not_reviewed(self, claims_list: List[Claim]):
        uuids = [claim.uuid for claim in claims_list]

        set_claims_status(uuids, 'review_status', Claim.REVIEW_BYPASSED)



from graphql_jwt.shortcuts import get_token
from django.test import TestCase
from unittest import mock

from claim.services import ClaimSubmitService
from core.test_helpers import create_test_interactive_user, create_test_officer
from location.test_helpers import create_test_location, create_test_health_facility, create_test_village
from insuree.test_helpers import create_test_insuree
from claim.test_helpers import create_test_claim_admin, create_test_claim, mark_test_claim_as_processed
from claim.models import Claim, ClaimItem, ClaimService, ClaimDetail
from medical.models import Diagnosis, Item, Service
from medical.test_helpers import create_test_item, create_test_service
from django.conf import settings
from .models import (
    ClaimSamplingBatch,
    ClaimSamplingBatchAssignment,
    ClaimSamplingBatchAssignmentStatus
)

from .services import ClaimSamplingService
import core
from core.models.openimis_graphql_test_case import openIMISGraphQLTestCase, BaseTestContext as DummyContext

from graphene import Schema
from graphene_django.utils.testing import GraphQLTestCase
from claim_sampling import schema as claim_schema
from graphene.test import Client
from policy.test_helpers import create_test_policy2
from product.test_helpers import create_test_product, create_test_product_service, create_test_product_item
from medical_pricelist.test_helpers import add_service_to_hf_pricelist, add_item_to_hf_pricelist
from product.models import ProductItemOrService
from datetime import date, timedelta, datetime
class ClaimSubmitServiceTestCase(openIMISGraphQLTestCase):
    GRAPHQL_URL = f'/{settings.SITE_ROOT()}graphql'
    # This is required by some version of graphene but is never used. It should be set to the schema but the import
    # is shown as an error in the IDE, so leaving it as True.
    GRAPHQL_SCHEMA = True

    test_hf = None

    test_insuree = None
    test_claim_admin = None
    test_icd = None
    test_claim = None
    test_claim_item = None
    test_claim_service = None
    test_region = None
    test_district = None
    test_village = None
    test_ward = None

    admin_user = None
    schema = None

    test_claims = []


    @classmethod
    def setUpTestData(cls):
        cls.admin_user = create_test_interactive_user(username="testLocationAdmin")
        cls.admin_token = get_token(cls.admin_user, DummyContext(user=cls.admin_user))
        cls.schema = Schema(
            query=claim_schema.Query,
            mutation=claim_schema.Mutation
        )
        cls.graph_client = Client(cls.schema)

        cls.officer = create_test_officer(custom_props={"code": "TSTSIMP1"})

        if cls.test_region is None:
            cls.test_village = create_test_village()
            cls.test_ward = cls.test_village.parent
            cls.test_region = cls.test_village.parent.parent.parent
            cls.test_district = cls.test_village.parent.parent

        cls.test_hf = create_test_health_facility("1", cls.test_district.id, valid=True)
        props = dict(
            last_name="name",
            other_names="surname",
            dob=core.datetime.date(2000, 1, 13),
            chf_id="884930485",
        )
        family_props = dict(
            location=cls.test_village,
        )
        cls.test_insuree = create_test_insuree(is_head=True, custom_props=props, family_custom_props=family_props)
        product = create_test_product("TEST_CLM")
        cls.test_policy , ip = create_test_policy2(product, cls.test_insuree)
        cls.test_claim_admin = create_test_claim_admin()
        cls.test_icd = Diagnosis(code='ICD00I', name='diag test', audit_user_id=-1)
        cls.test_icd.save()

        cls._create_test_claims(cls.test_policy.product)
        

    @classmethod
    def _create_test_claims(cls, product):
        configdate = datetime.now() - timedelta(days=265)

        test_item = create_test_item(
            'D',
            custom_props={"code": "csamCo", "price": 1000 }
        )
        test_service = create_test_service(
            'D',
            custom_props={"code": "ssamCo", "price": 1000}
        )
        create_test_product_service(
            product,
            test_service,
            custom_props={"price_origin": ProductItemOrService.ORIGIN_RELATIVE},
        )
        create_test_product_item(
            product,
            test_item,
            custom_props={"price_origin": ProductItemOrService.ORIGIN_RELATIVE},
        )
        add_service_to_hf_pricelist(test_service, hf_id=cls.test_hf.id)
        add_item_to_hf_pricelist(test_item, hf_id=cls.test_hf.id)
        
        dateclaim = date.today() - timedelta(days=5)
        datetimeclaim = datetime.now() - timedelta(days=5)
        for i in range(10):
            claim = Claim.objects.create(
                date_claimed=dateclaim,
                code=F"code_ABV{i}",
                icd=cls.test_icd,
                claimed=2000,
                date_from=dateclaim,
                date_to=None,
                admin=cls.test_claim_admin,
                insuree=cls.test_insuree,
                health_facility=cls.test_hf,
                status=Claim.STATUS_ENTERED,
                audit_user_id=-1,
                validity_from=datetimeclaim
            )
            claim_item = ClaimItem.objects.create(
                claim=claim,
                item=test_item,
                price_asked=1000,
                qty_provided=1,
                audit_user_id=-1,
                status=ClaimDetail.STATUS_PASSED,
                availability=True,
                validity_from=datetimeclaim
            )
            claim_service = ClaimService.objects.create(
                claim=claim,
                service=test_service,
                price_asked=1000,
                qty_provided=1,
                audit_user_id=-1,
                status=ClaimDetail.STATUS_PASSED,
                validity_from=datetimeclaim
            )
            claim.refresh_from_db()
            ClaimSubmitService(cls.admin_user).submit_claim(claim)
            cls.test_claims.append(claim)
        
    @classmethod
    def _set_claim_as_valuated(cls, claim, user, is_process=False):
        # Mock of dedrem
        claim.status = Claim.STATUS_PROCESSED
        claim.save()
        return []

    def test_mutation_create_claim( self):
        
        percentage_for_sample = 20
        mutation = f'''
mutation {{
  createClaimSamplingBatch(
    input: {{
      clientMutationId: "fdcc211f-7225-4f0e-8a66-11223344667d"
      clientMutationLabel: "Create Claim Sampling Batch" 
      percentage: 30
      filters: "{{\\"status\\":4, \\"dateFrom\\": \\"{date.today() - timedelta(days=5)}\\"}}"        
    }}      
  ) {{
    clientMutationId
    internalId
  }}    
}}
            '''
        response = self.send_mutation_raw(mutation,self.admin_token )
        

        claim_sampling = ClaimSamplingBatch.objects.first()
        self.assertIsNotNone(claim_sampling)

        attachments = ClaimSamplingBatchAssignment.objects.filter(claim_batch=claim_sampling)
        # Ten claims, 2 should be assigned for sample idle and 8 for skip;
        idle = list(attachments.filter(status=ClaimSamplingBatchAssignmentStatus.IDLE))
        skip = list(attachments.filter(status=ClaimSamplingBatchAssignmentStatus.SKIPPED))

        # Creation
        self.assertEqual(len(idle), 3)
        self.assertEqual(len(skip), 7)
        self.assertEqual(idle[0].claim.review_status, Claim.REVIEW_SELECTED)
        self.assertEqual(idle[1].claim.review_status, Claim.REVIEW_SELECTED)
        self.assertEqual(idle[2].claim.review_status, Claim.REVIEW_SELECTED)

        # Summary
        claim_1, claim_2, claim_3 = idle[0].claim, idle[1].claim, idle[2].claim
        claim_1.review_status = Claim.REVIEW_DELIVERED
        claim_1.status = Claim.STATUS_PROCESSED
        claim_1.save()

        claim_2.review_status = Claim.REVIEW_DELIVERED
        claim_2.status = Claim.STATUS_REJECTED
        claim_2.save()

        claim_3.review_status = Claim.REVIEW_DELIVERED
        claim_3.status = Claim.STATUS_REJECTED
        claim_3.save()

        service = ClaimSamplingService(self.admin_user)
        rejected_from_review, reviewed_delivered, total = service.prepare_sampling_summary(claim_sampling.id)
        self.assertEqual(rejected_from_review.count(), 2)
        self.assertEqual(reviewed_delivered.count(), 3)
        self.assertEqual(total, 3)
        datetimeclaim = datetime.now() - timedelta(days=5)

        # Extrapolation
        service.extrapolate_results(claim_sampling.id)
        attachments = ClaimSamplingBatchAssignment.objects.filter(claim_batch=claim_sampling)
        # 50% of remaining claims should be rejected and 50% should be valuated
        skip = [x.claim for x in attachments.filter(status=ClaimSamplingBatchAssignmentStatus.SKIPPED)]
        accepted = [x for x in skip if x.status in [ Claim.STATUS_PROCESSED, Claim.STATUS_VALUATED]]
        rejected = [x for x in skip if x.status in [Claim.STATUS_REJECTED]]
        self.assertEqual(len(accepted), 7)
        self.assertEqual(len(rejected), 0)
        
        # FIXME check the ratio (done manually looks ok)
        

    def _get_test_dict(self, code=None):
        return {
            "health_facility_id": self.test_claim.health_facility_id,
            "icd_id": self.test_icd.id,
            "date_from": self.test_claim.date_from,
            "code": self.test_claim.code if code is None else code,
            "date_claimed": self.test_claim.date_claimed,
            "date_to": self.test_claim.date_to,
            "audit_user_id": self.test_claim.audit_user_id,
            "insuree_id": self.test_claim.insuree_id,
            "status": self.test_claim.status,
            "validity_from": self.test_claim.validity_from,
            "items": [{
                "qty_provided": self.test_claim_item.qty_provided,
                "price_asked": self.test_claim_item.price_asked,
                "item_id": self.test_claim_item.item_id,
                "status": self.test_claim_item.status,
                "availability": self.test_claim_item.availability,
                "validity_from": self.test_claim_item.validity_from,
                "validity_to": self.test_claim_item.validity_to,
                "audit_user_id": self.test_claim_item.audit_user_id
            }],
            "services": [{
                "qty_provided": self.test_claim_service.qty_provided,
                "price_asked": self.test_claim_service.price_asked,
                "service_id": self.test_claim_service.service_id,
                "status": self.test_claim_service.status,
                "validity_from": self.test_claim_service.validity_from,
                "validity_to": self.test_claim_service.validity_to,
                "audit_user_id": self.test_claim_service.audit_user_id
            }]
        }



# Generated by Django 3.2.16 on 2023-10-25 07:50

import core.fields
import datetime
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('claim', '0024_add_claim_restore_perms'),
    ]

    operations = [
        migrations.CreateModel(
            name='ClaimSamplingBatch',
            fields=[
                ('validity_from', core.fields.DateTimeField(db_column='ValidityFrom', default=datetime.datetime.now)),
                ('validity_to', core.fields.DateTimeField(blank=True, db_column='ValidityTo', null=True)),
                ('legacy_id', models.IntegerField(blank=True, db_column='LegacyID', null=True)),
                ('id', models.AutoField(db_column='ClaimSamplingBatchID', primary_key=True, serialize=False)),
                ('uuid', models.CharField(db_column='ClaimSamplingBatchUUID', default=uuid.uuid4, max_length=36, unique=True)),
                ('is_completed', models.BooleanField()),
                ('is_applied', models.BooleanField()),
                ('computed_value', models.JSONField(blank=True, db_column='ComputedValue', null=True)),
                ('assigned_value', models.JSONField(blank=True, db_column='AssignedValue', null=True)),
                ('assigned_to', models.ForeignKey(db_column='AssignedTo', on_delete=django.db.models.deletion.DO_NOTHING, related_name='AssignedTo', to='claim.claimadmin')),
                ('created_by', models.ForeignKey(db_column='CreatedBy', on_delete=django.db.models.deletion.DO_NOTHING, related_name='CreatedBy', to='claim.claimadmin')),
            ],
            options={
                'db_table': 'claim_ClaimSamplingBatch',
                'managed': True,
            },
        ),
        migrations.CreateModel(
            name='ClaimSamplingBatchAssignment',
            fields=[
                ('validity_from', core.fields.DateTimeField(db_column='ValidityFrom', default=datetime.datetime.now)),
                ('validity_to', core.fields.DateTimeField(blank=True, db_column='ValidityTo', null=True)),
                ('legacy_id', models.IntegerField(blank=True, db_column='LegacyID', null=True)),
                ('id', models.AutoField(db_column='ClaimSamplingBatchAssignmentID', primary_key=True, serialize=False)),
                ('uuid', models.CharField(db_column='ClaimSamplingBatchAssignmentUUID', default=uuid.uuid4, max_length=36, unique=True)),
                ('status', models.CharField(choices=[('R', 'Reviewed'), ('S', 'Skipped'), ('I', 'Idle')], default='I', max_length=2)),
                ('claim_batch_id', models.ForeignKey(db_column='ClaimSamplingBatchID', on_delete=django.db.models.deletion.DO_NOTHING, related_name='ClaimSamplingBatchID', to='claim_sampling.claimsamplingbatch')),
                ('claim_id', models.ForeignKey(db_column='ClaimID', on_delete=django.db.models.deletion.DO_NOTHING, related_name='ClaimID', to='claim.claim')),
            ],
            options={
                'db_table': 'claim_ClaimBatchAssignment',
                'managed': True,
            },
        ),
    ]



from enum import Enum

from django.db.models import OuterRef, Subquery, Avg, Q
import graphene_django_optimizer as gql_optimizer
from core.schema import OrderedDjangoFilterConnectionField
from core import filter_validity
from django.conf import settings
from claim_sampling.gql_queries import ClaimSamplingSummaryGQLType, ClaimSamplingBatchGQLType, ClaimSamplingBatchAssignmentGQLType
from django.utils.translation import gettext as _
from claim_sampling.gql_mutations import *  # lgtm [py/polluting-import]
from django.core.exceptions import PermissionDenied

from claim_sampling.models import ClaimSamplingBatch, ClaimSamplingBatchAssignment
from claim.models import Claim
from tasks_management.models import Task


class Query(graphene.ObjectType):
    claim_sampling_batch = OrderedDjangoFilterConnectionField(
        ClaimSamplingBatchGQLType,
        id=graphene.Int(required=True)
    )

    claim_sampling_batch_assignment = graphene.Field(
        ClaimSamplingBatchAssignmentGQLType,
        id=graphene.Int(required=True)
    )

    sampling_batch_claims = OrderedDjangoFilterConnectionField(
        ClaimGQLType,
        claim_sampling_id=graphene.UUID(required=True),
        assignment_status=graphene.String()
    )

    sampling_summary = graphene.Field(
        ClaimSamplingSummaryGQLType,
        task_id=graphene.String(required=True),
        description="Provide details regarding claim sampling assigned to specific task."
    )

    def resolve_claim_sampling_batch(self, info, **kwargs):
        if (
            not info.context.user.has_perms(ClaimSamplingConfig.gql_query_claim_batch_samplings_perms)
            and settings.ROW_SECURITY
        ):
            raise PermissionDenied(_("unauthorized"))

        claim_sampling_batch_id = kwargs.get("id", None)

        return ClaimSamplingBatch.objects.get(id=claim_sampling_batch_id, validity_to__isnull=True)

    def resolve_claim_sampling_batch_assignment(self, info, **kwargs):
        if (
            not info.context.user.has_perms(ClaimSamplingConfig.gql_query_claim_batch_samplings_perms)
            and settings.ROW_SECURITY
        ):
            raise PermissionDenied(_("unauthorized"))

        claim_sampling_batch_assignment_id = kwargs.get("id", None)

        return ClaimSamplingBatchAssignment.objects.get(id=claim_sampling_batch_assignment_id, validity_to__isnull=True)

    def resolve_sampling_batch_claims(self, info, **kwargs):
        if (
            not info.context.user.has_perms(ClaimSamplingConfig.gql_query_claim_batch_samplings_perms)
            and settings.ROW_SECURITY
        ):
            raise PermissionDenied(_("unauthorized"))

        sampling = ClaimSamplingBatch.objects.get(uuid=kwargs['claim_sampling_id'])
        relevant_claims = ClaimSamplingBatchAssignment.objects.filter(claim_batch=sampling)

        claim_assignment_status = kwargs.get('assignment_status')

        if claim_assignment_status:
            relevant_claims = relevant_claims.filter(status=claim_assignment_status)

        # All claims are displayed but
        query = Claim.objects.filter(
            id__in=relevant_claims.values_list('claim_id', flat=True),
            validity_to__isnull=True  # Ensuring that only valid (non-expired) claims are returned
        ).order_by('status')

        return query

    def resolve_sampling_summary(self, info, **kwargs):
        if not info.context.user.has_perms(ClaimSamplingConfig.gql_query_claim_batch_samplings_perms):
            raise PermissionDenied(_("unauthorized"))

        try:
            task_id = kwargs['task_id']
            task = Task.objects.get(id=task_id)
            claim_sampling_id = task.data['data']['uuid']

            claim_sampling_service = ClaimSamplingService(user=info.context.user)
            rejected_from_review, reviewed_delivered, total = claim_sampling_service.prepare_sampling_summary(
                claim_sampling_id)

            review_delivered = round(reviewed_delivered.count()/total, 2)*100
            percentage = round(rejected_from_review.count()/total, 2)*100

            return ClaimSamplingSummaryGQLType(
                deductible_percentage=percentage,
                reviewed_percentage=review_delivered,
                total_claims_in_batch=total
            )
        except Exception as e:
            if settings.DEBUG:
                import traceback
                traceback.print_exc()
            raise e


class Mutation(graphene.ObjectType):
    create_claim_sampling_batch = CreateClaimSamplingBatchMutation.Field()
    update_claim_sampling_batch = UpdateClaimSamplingBatchMutation.Field()
    approve_claim_sampling_batch = ApproveClaimSamplingBatchMutation.Field()



from django.db import models
from claim.models import Claim
from core.models import HistoryModel


class ClaimSamplingBatch(HistoryModel):
    is_completed = models.BooleanField()
    is_applied = models.BooleanField()
    computed_value = models.JSONField(db_column="ComputedValue", blank=True, null=True)
    assigned_value = models.JSONField(db_column="AssignedValue", blank=True, null=True)

    def __str__(self):
        return f"Claim Sampling - {self.date_created}"


class ClaimSamplingBatchAssignmentStatus(models.TextChoices):
    SKIPPED = "S"  # Claims Which Validation is based on sampling
    IDLE = "I"  # Part of the sample


class ClaimSamplingBatchAssignment(HistoryModel):

    claim = models.ForeignKey(Claim, models.DO_NOTHING, db_column='ClaimID', related_name="assignments")
    claim_batch = models.ForeignKey(ClaimSamplingBatch, models.DO_NOTHING, db_column='ClaimSamplingBatchID',
                                    related_name="assignments")
    status = models.CharField(
        max_length=2,
        choices=ClaimSamplingBatchAssignmentStatus.choices,
        default=ClaimSamplingBatchAssignmentStatus.IDLE
    )



# Generated by Django 4.2.11 on 2024-05-19 15:10

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("claim_sampling", "0003_initial"),
    ]

    operations = [
        migrations.RenameField(
            model_name="claimsamplingbatchassignment",
            old_name="claim_id",
            new_name="claim",
        ),
        migrations.RenameField(
            model_name="claimsamplingbatchassignment",
            old_name="claim_batch_id",
            new_name="claim_batch",
        ),
        migrations.RenameField(
            model_name="historicalclaimsamplingbatchassignment",
            old_name="claim_id",
            new_name="claim",
        ),
        migrations.RenameField(
            model_name="historicalclaimsamplingbatchassignment",
            old_name="claim_batch_id",
            new_name="claim_batch",
        ),
    ]



# Generated by Django 4.2.10 on 2024-08-20 08:08

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("claim_sampling", "0005_alter_claimsamplingbatchassignment_status_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="claimsamplingbatchassignment",
            name="claim",
            field=models.ForeignKey(
                db_column="ClaimID",
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="assignments",
                to="claim.claim",
            ),
        ),
        migrations.AlterField(
            model_name="claimsamplingbatchassignment",
            name="claim_batch",
            field=models.ForeignKey(
                db_column="ClaimSamplingBatchID",
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="assignments",
                to="claim_sampling.claimsamplingbatch",
            ),
        ),
    ]



import graphene
from core import prefix_filterset, ExtendedConnection
from graphene_django import DjangoObjectType
from .apps import ClaimSamplingConfig
from .models import Claim
from django.utils.translation import gettext as _
from django.core.exceptions import PermissionDenied


class ClaimSamplingBatchGQLType(DjangoObjectType):
    attachments_count = graphene.Int()
    client_mutation_id = graphene.String()

    class Meta:
        model = Claim
        interfaces = (graphene.relay.Node,)
        filter_fields = {
            "uuid": ["exact"],
            "code": ["exact", "istartswith", "icontains", "iexact"],
            "status": ["exact", "gt"],
            "date_claimed": ["exact", "lt", "lte", "gt", "gte"],
            "date_from": ["exact", "lt", "lte", "gt", "gte"],
            "date_to": ["exact", "lt", "lte", "gt", "gte"],
            "date_processed": ["exact", "lt", "lte", "gt", "gte"],
            "feedback_status": ["exact"],
            "review_status": ["exact"],
            "claimed": ["exact", "lt", "lte", "gt", "gte"],
            "approved": ["exact", "lt", "lte", "gt", "gte"],
            "visit_type": ["exact"],
            "attachments_count__value": ["exact", "lt", "lte", "gt", "gte"],
        }
        connection_class = ExtendedConnection

    def resolve_client_mutation_id(self, info):
        if not info.context.user.has_perms(ClaimSamplingConfig.gql_query_claim_batch_samplings_perms):
            raise PermissionDenied(_("unauthorized"))
        claim_mutation = self.mutations.select_related(
            'mutation').filter(mutation__status=0).first()
        return claim_mutation.mutation.client_mutation_id if claim_mutation else None

    @classmethod
    def get_queryset(cls, queryset, info):
        claim_ids = Claim.get_queryset(queryset, info).values('uuid').all()
        return Claim.objects.filter(uuid__in=claim_ids)


class ClaimSamplingBatchAssignmentGQLType(DjangoObjectType):
    """
    Main element for a Claim. It can contain items and/or services.
    The filters are possible on BatchRun, Insuree, HealthFacility, Admin and ICD in addition to the Claim fields
    themselves.
    """

    class Meta:
        model = Claim
        interfaces = (graphene.relay.Node,)
        filter_fields = {
            "uuid": ["exact"],
            "code": ["exact", "istartswith", "icontains", "iexact"],
            "status": ["exact", "gt"],
            "date_claimed": ["exact", "lt", "lte", "gt", "gte"],
            "date_from": ["exact", "lt", "lte", "gt", "gte"],
            "date_to": ["exact", "lt", "lte", "gt", "gte"],
            "date_processed": ["exact", "lt", "lte", "gt", "gte"],
            "feedback_status": ["exact"],
            "review_status": ["exact"],
            "claimed": ["exact", "lt", "lte", "gt", "gte"],
            "approved": ["exact", "lt", "lte", "gt", "gte"],
            "visit_type": ["exact"],
            "attachments_count__value": ["exact", "lt", "lte", "gt", "gte"],
        }
        connection_class = ExtendedConnection


class ClaimSamplingSummaryGQLType(graphene.ObjectType):
    deductible_percentage = graphene.Float(description="Percentage of claims rejected during review - deductibles.")
    reviewed_percentage = graphene.Float(description="Percentage of reviewed claims in batch.")
    total_claims_in_batch = graphene.Int(description="Total number of claims selected for review in sampling batch")




# Generated by Django 4.2.10 on 2024-06-11 12:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('claim_sampling', '0004_rename_claim_id_claimsamplingbatchassignment_claim_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='claimsamplingbatchassignment',
            name='status',
            field=models.CharField(choices=[('S', 'Skipped'), ('I', 'Idle')], default='I', max_length=2),
        ),
        migrations.AlterField(
            model_name='historicalclaimsamplingbatchassignment',
            name='status',
            field=models.CharField(choices=[('S', 'Skipped'), ('I', 'Idle')], default='I', max_length=2),
        ),
    ]



from django.apps import AppConfig

MODULE_NAME = 'claim_sampling'

DEFAULT_CFG = {
    "gql_query_claim_batch_samplings_perms": ["126001"],
    "gql_mutation_create_claim_batch_samplings_perms": ["126002"],
    "gql_mutation_update_claim_batch_samplings_perms": ["126003"],
    "gql_mutation_approve_claim_batch_samplings_perms": ["126004"],
}


class ClaimSamplingConfig(AppConfig):
    name = MODULE_NAME

    gql_query_claim_batch_samplings_perms = None
    gql_mutation_create_claim_batch_samplings_perms = None
    gql_mutation_update_claim_batch_samplings_perms = None
    gql_mutation_approve_claim_batch_samplings_perms = None

    def __load_config(self, cfg):
        for field in cfg:
            if hasattr(ClaimSamplingConfig, field):
                setattr(ClaimSamplingConfig, field, cfg[field])

    def ready(self):
        from core.models import ModuleConfiguration
        cfg = ModuleConfiguration.get_or_default(MODULE_NAME, DEFAULT_CFG)
        self.__load_config(cfg)



# Generated by Django 4.2.11 on 2024-05-19 14:49

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("claim_sampling", "0001_initial"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="claimsamplingbatchassignment",
            name="claim_batch_id",
        ),
        migrations.RemoveField(
            model_name="claimsamplingbatchassignment",
            name="claim_id",
        ),
        migrations.DeleteModel(
            name="ClaimSamplingBatch",
        ),
        migrations.DeleteModel(
            name="ClaimSamplingBatchAssignment",
        ),
    ]
